@@ -62,6 +62,16 @@ typedef enum
 }
 mlt_audio_format;
 
+/** The time string formats */
+
+typedef enum
+{
+	mlt_time_frames = 0, /**< frame count */
+	mlt_time_clock,      /**< SMIL clock-value as [[hh:]mm:]ss[.fraction] */
+	mlt_time_smpte       /**< SMPTE timecode as [[[hh:]mm:]ss:]frames */
+}
+mlt_time_format;
+
 /** The relative time qualifiers */
 
 typedef enum
