@@ -1,6 +1,6 @@
 /*
  * melt.c -- MLT command line utility
- * Copyright (C) 2002-2011 Ushodaya Enterprises Limited
+ * Copyright (C) 2002-2012 Ushodaya Enterprises Limited
  * Authors: Charles Yates <charles.yates@pandora.be>
  *          Dan Dennedy <dan@dennedy.org>
  *
@@ -262,7 +262,49 @@ static mlt_consumer create_consumer( mlt_profile profile, char *id )
 static void load_consumer( mlt_consumer *consumer, mlt_profile profile, int argc, char **argv )
 {
 	int i;
+	int multi = 0;
+
 	for ( i = 1; i < argc; i ++ )
+		multi += !strcmp( argv[ i ], "-consumer" );
+
+	if ( multi > 1 )
+	{
+		// If there is more than one -consumer use the 'multi' consumer.
+		int k = 0;
+		char key[20];
+
+		if ( *consumer )
+			mlt_consumer_close( *consumer );
+		*consumer = create_consumer( profile, "multi" );
+		mlt_properties properties = MLT_CONSUMER_PROPERTIES( *consumer );
+		for ( i = 1; i < argc; i ++ )
+		{
+			if ( !strcmp( argv[ i ], "-consumer" ) && argv[ i + 1 ])
+			{
+				// Create a properties object for each sub-consumer
+				mlt_properties new_props = mlt_properties_new();
+				snprintf( key, sizeof(key), "%d", k++ );
+				mlt_properties_set_data( properties, key, new_props, 0,
+					(mlt_destructor) mlt_properties_close, NULL );
+				if ( strchr( argv[i + 1], ':' ) )
+				{
+					char *temp = strdup( argv[++i] );
+					char *service = temp;
+					char *target = strchr( temp, ':' );
+					*target++ = 0;
+					mlt_properties_set( new_props, "mlt_service", service );
+					mlt_properties_set( new_props, "target", target );
+				}
+				else
+				{
+					mlt_properties_set( new_props, "mlt_service", argv[ ++i ] );
+				}
+				while ( argv[ i + 1 ] && strchr( argv[ i + 1 ], '=' ) )
+					mlt_properties_parse( new_props, argv[ ++ i ] );
+			}
+		}
+	}
+	else for ( i = 1; i < argc; i ++ )
 	{
 		if ( !strcmp( argv[ i ], "-consumer" ) )
 		{
@@ -272,7 +314,7 @@ static void load_consumer( mlt_consumer *consumer, mlt_profile profile, int argc
 			if ( *consumer )
 			{
 				mlt_properties properties = MLT_CONSUMER_PROPERTIES( *consumer );
-				while ( argv[ i + 1 ] != NULL && strstr( argv[ i + 1 ], "=" ) )
+				while ( argv[ i + 1 ] != NULL && strchr( argv[ i + 1 ], '=' ) )
 					mlt_properties_parse( properties, argv[ ++ i ] );
 			}
 		}
@@ -727,8 +769,8 @@ query_all:
 		}
 		else if ( !strcmp( argv[ i ], "-version" ) || !strcmp( argv[ i ], "--version" ) )
 		{
-			fprintf( stdout, "MLT %s " VERSION "\n"
-				"Copyright (C) 2002-2011 Ushodaya Enterprises Limited\n"
+			fprintf( stdout, "%s " VERSION "\n"
+				"Copyright (C) 2002-2012 Ushodaya Enterprises Limited\n"
 				"<http://www.mltframework.org/>\n"
 				"This is free software; see the source for copying conditions.  There is NO\n"
 				"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
