@@ -316,3 +316,23 @@ char *Properties::serialise_yaml( )
 {
 	return mlt_properties_serialise_yaml( get_properties( ) );
 }
+
+int Properties::preset( const char *name )
+{
+	return mlt_properties_preset( get_properties(), name );
+}
+
+int Properties::set_lcnumeric( const char *locale )
+{
+	return mlt_properties_set_lcnumeric( get_properties(), locale );
+}
+
+const char *Properties::get_lcnumeric( )
+{
+	return mlt_properties_get_lcnumeric( get_properties() );
+}
+
+char *Properties::get_time( const char *name, mlt_time_format format )
+{
+	return mlt_properties_get_time( get_properties(), name, format );
+}
