@@ -101,7 +101,6 @@ struct producer_avformat_s
 	int seekable;
 	int64_t current_position;
 	mlt_position nonseek_position;
-	int got_picture;
 	int top_field_first;
 	uint8_t *audio_buffer[ MAX_AUDIO_STREAMS ];
 	size_t audio_buffer_size[ MAX_AUDIO_STREAMS ];
@@ -328,6 +327,10 @@ static mlt_properties find_default_streams( producer_avformat self )
 				snprintf( key, sizeof(key), "meta.media.%d.stream.sample_aspect_ratio", i );
 				mlt_properties_set_double( meta_media, key, av_q2d( context->streams[ i ]->sample_aspect_ratio ) );
 #endif
+				snprintf( key, sizeof(key), "meta.media.%d.codec.width", i );
+				mlt_properties_set_int( meta_media, key, codec_context->width );
+				snprintf( key, sizeof(key), "meta.media.%d.codec.height", i );
+				mlt_properties_set_int( meta_media, key, codec_context->height );
 				snprintf( key, sizeof(key), "meta.media.%d.codec.frame_rate", i );
 				mlt_properties_set_double( meta_media, key, (double) codec_context->time_base.den /
 										   ( codec_context->time_base.num == 0 ? 1 : codec_context->time_base.num ) );
@@ -361,6 +364,8 @@ static mlt_properties find_default_streams( producer_avformat self )
 #endif
 				break;
 			case CODEC_TYPE_AUDIO:
+				if ( !codec_context->channels )
+					break;
 				// Use first audio stream
 				if ( self->audio_index < 0 )
 					self->audio_index = i;
@@ -1079,6 +1084,8 @@ static int seek_video( producer_avformat self, mlt_position position,
 					timestamp += self->first_pts;
 				else if ( context->start_time != AV_NOPTS_VALUE )
 					timestamp += context->start_time;
+				if ( req_position <= 0 )
+					timestamp = 0;
 			}
 			else
 			{
@@ -1243,7 +1250,7 @@ static mlt_image_format pick_format( enum PixelFormat pix_fmt )
 }
 
 static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
-	mlt_image_format *format, int width, int height, int colorspace )
+	mlt_image_format *format, int width, int height, int colorspace, uint8_t **alpha )
 {
 #ifdef SWSCALE
 	int full_range = -1;
@@ -1256,6 +1263,25 @@ static void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 	flags |= SWS_CPU_CAPS_MMX2;
 #endif
 
+	// extract alpha from planar formats
+	if ( ( pix_fmt == PIX_FMT_YUVA420P
+#if LIBAVUTIL_VERSION_INT >= ((51<<16)+(35<<8)+101)
+			|| pix_fmt == PIX_FMT_YUVA444P
+#endif
+			) &&
+		*format != mlt_image_rgb24a && *format != mlt_image_opengl &&
+		frame->data[3] && frame->linesize[3] )
+	{
+		int i;
+		uint8_t *src, *dst;
+
+		dst = *alpha = mlt_pool_alloc( width * height );
+		src = frame->data[3];
+
+		for ( i = 0; i < height; dst += width, src += frame->linesize[3], i++ )
+			memcpy( dst, src, FFMIN( width, frame->linesize[3] ) );
+	}
+
 	if ( *format == mlt_image_yuv420p )
 	{
 		struct SwsContext *context = sws_getContext( width, height, pix_fmt,
@@ -1388,15 +1414,31 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	// Get codec context
 	AVCodecContext *codec_context = stream->codec;
 
+	uint8_t *alpha = NULL;
+	int got_picture = 0;
+	int image_size = 0;
+
 	// Get the image cache
 	if ( ! self->image_cache && ! mlt_properties_get_int( properties, "noimagecache" ) )
+	{
 		self->image_cache = mlt_cache_init();
+	}
 	if ( self->image_cache )
 	{
-		mlt_cache_item item = mlt_cache_get( self->image_cache, (void*) position );
-		uint8_t *original = mlt_cache_item_data( item, (int*) format );
+		mlt_frame original = mlt_cache_get_frame( self->image_cache, mlt_frame_get_position( frame ) );
 		if ( original )
 		{
+			mlt_properties orig_props = MLT_FRAME_PROPERTIES( original );
+			int size = 0;
+
+			*buffer = mlt_properties_get_data( orig_props, "alpha", &size );
+			if (*buffer)
+				mlt_frame_set_alpha( frame, *buffer, size, NULL );
+			*buffer = mlt_properties_get_data( orig_props, "image", &size );
+			mlt_frame_set_image( frame, *buffer, size, NULL );
+			mlt_properties_set_data( frame_properties, "avformat.image_cache", original, 0, (mlt_destructor) mlt_frame_close, NULL );
+			*format = mlt_properties_get_int( orig_props, "format" );
+
 			// Set the resolution
 			*width = codec_context->width;
 			*height = codec_context->height;
@@ -1405,28 +1447,11 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 			if ( *height == 1088 && mlt_profile_dar( mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) ) ) == 16.0/9.0 )
 				*height = 1080;
 
-			// Cache hit
-			int size = mlt_image_format_size( *format, *width, *height, NULL );
-			if ( writable )
-			{
-				*buffer = mlt_pool_alloc( size );
-				mlt_frame_set_image( frame, *buffer, size, mlt_pool_release );
-				memcpy( *buffer, original, size );
-				mlt_cache_item_close( item );
-			}
-			else
-			{
-				*buffer = original;
-				mlt_properties_set_data( frame_properties, "avformat.image_cache", item, 0, ( mlt_destructor )mlt_cache_item_close, NULL );
-				mlt_frame_set_image( frame, *buffer, size, NULL );
-			}
-			self->got_picture = 1;
-
+			got_picture = 1;
 			goto exit_get_image;
 		}
 	}
 	// Cache miss
-	int image_size = 0;
 
 	// Packet
 	AVPacket pkt;
@@ -1458,13 +1483,18 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	double delay = mlt_properties_get_double( properties, "video_delay" );
 
 	// Seek if necessary
-	int paused = seek_video( self, position, req_position, must_decode, use_new_seek, &ignore );
+	const char *interp = mlt_properties_get( frame_properties, "rescale.interp" );
+	int preseek = must_decode
+#if defined(FFUDIV) && LIBAVFORMAT_VERSION_INT >= ((53<<16)+(24<<8)+2)
+		&& ( !use_new_seek || ( interp && strcmp( interp, "nearest" ) ) )
+#endif
+		&& codec_context->has_b_frames;
+	int paused = seek_video( self, position, req_position, preseek, use_new_seek, &ignore );
 
 	// Seek might have reopened the file
 	context = self->video_format;
 	stream = context->streams[ self->video_index ];
 	codec_context = stream->codec;
-
 	if ( *format == mlt_image_none ||
 			codec_context->pix_fmt == PIX_FMT_ARGB ||
 			codec_context->pix_fmt == PIX_FMT_RGBA ||
@@ -1473,10 +1503,8 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		*format = pick_format( codec_context->pix_fmt );
 
 	// Duplicate the last image if necessary
-	if ( self->av_frame && self->av_frame->linesize[0] && self->got_picture
-		 && ( paused
-			  || self->current_position == req_position
-			  || ( !use_new_seek && self->current_position > req_position ) ) )
+	if ( self->av_frame && self->av_frame->linesize[0]
+		 && ( paused || self->current_position >= req_position ) )
 	{
 		// Duplicate it
 		if ( ( image_size = allocate_buffer( frame, codec_context, buffer, format, width, height ) ) )
@@ -1495,22 +1523,20 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				picture.linesize[1] = codec_context->width / 2;
 				picture.linesize[2] = codec_context->width / 2;
 				convert_image( (AVFrame*) &picture, *buffer,
-					PIX_FMT_YUV420P, format, *width, *height, self->colorspace );
+					PIX_FMT_YUV420P, format, *width, *height, self->colorspace, &alpha );
 			}
 			else
 #endif
 			convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-				format, *width, *height, self->colorspace );
+				format, *width, *height, self->colorspace, &alpha );
+			got_picture = 1;
 		}
-		else
-			mlt_frame_get_image( frame, buffer, format, width, height, writable );
 	}
 	else
 	{
 		int ret = 0;
 		int64_t int_position = 0;
 		int decode_errors = 0;
-		int got_picture = 0;
 
 		av_init_packet( &pkt );
 
@@ -1637,7 +1663,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 							pts -= self->first_pts;
 						else if ( context->start_time != AV_NOPTS_VALUE )
 							pts -= context->start_time;
-						int_position = ( int64_t )( av_q2d( stream->time_base) * pts * source_fps + 0.1 );
+						int_position = ( int64_t )( ( av_q2d( stream->time_base) * pts + delay ) * source_fps + 0.1 );
 						mlt_log_debug( MLT_PRODUCER_SERVICE(producer), "got frame %"PRId64", key %d\n", int_position, self->av_frame->key_frame );
 					}
 					// Handle ignore
@@ -1690,7 +1716,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 							if ( status == VDP_STATUS_OK )
 							{
 								convert_image( self->av_frame, *buffer, PIX_FMT_YUV420P,
-									format, *width, *height, self->colorspace );
+									format, *width, *height, self->colorspace, &alpha );
 							}
 							else
 							{
@@ -1707,10 +1733,9 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 					else
 #endif
 					convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-						format, *width, *height, self->colorspace );
+						format, *width, *height, self->colorspace, &alpha );
 					self->top_field_first |= self->av_frame->top_field_first;
 					self->current_position = int_position;
-					self->got_picture = 1;
 				}
 				else
 				{
@@ -1722,15 +1747,20 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		}
 	}
 
-	if ( self->got_picture && image_size > 0 && self->image_cache )
+	// set alpha
+	if ( alpha )
+		mlt_frame_set_alpha( frame, alpha, (*width) * (*height), mlt_pool_release );
+
+	if ( image_size > 0 && self->image_cache )
 	{
-		// Copy buffer to image cache	
-		uint8_t *image = mlt_pool_alloc( image_size );
-		memcpy( image, *buffer, image_size );
-		mlt_cache_put( self->image_cache, (void*) position, image, *format, mlt_pool_release );
+		mlt_properties_set_int( frame_properties, "format", *format );
+		mlt_cache_put_frame( self->image_cache, frame );
 	}
+
 	// Try to duplicate last image if there was a decoding failure
-	else if ( !image_size && self->av_frame && self->av_frame->linesize[0] )
+	// TODO: with multithread decoding a partial frame decoding resulting
+	// in failure also resets av_frame making test below fail.
+	if ( !image_size && self->av_frame && self->av_frame->linesize[0] )
 	{
 		// Duplicate it
 		if ( ( image_size = allocate_buffer( frame, codec_context, buffer, format, width, height ) ) )
@@ -1749,16 +1779,14 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 				picture.linesize[1] = codec_context->width / 2;
 				picture.linesize[2] = codec_context->width / 2;
 				convert_image( (AVFrame*) &picture, *buffer,
-					PIX_FMT_YUV420P, format, *width, *height, self->colorspace );
+					PIX_FMT_YUV420P, format, *width, *height, self->colorspace, &alpha );
 			}
 			else
 #endif
 			convert_image( self->av_frame, *buffer, codec_context->pix_fmt,
-				format, *width, *height, self->colorspace );
-			self->got_picture = 1;
+				format, *width, *height, self->colorspace, &alpha );
+			got_picture = 1;
 		}
-		else
-			mlt_frame_get_image( frame, buffer, format, width, height, writable );
 	}
 
 	// Regardless of speed, we expect to get the next frame (cos we ain't too bright)
@@ -1786,7 +1814,7 @@ exit_get_image:
 	mlt_properties_set_int( properties, "meta.media.progressive", mlt_properties_get_int( frame_properties, "progressive" ) );
 	mlt_service_unlock( MLT_PRODUCER_SERVICE( producer ) );
 
-	return !self->got_picture;
+	return !got_picture;
 }
 
 /** Process properties as AVOptions and apply to AV context obj
@@ -1936,7 +1964,6 @@ static int video_codec_init( producer_avformat self, int index, mlt_properties p
 #endif
 				mlt_properties_set_int( properties, "meta.media.frame_rate_num", frame_rate.num );
 				mlt_properties_set_int( properties, "meta.media.frame_rate_den", frame_rate.den );
-				source_fps = av_q2d( frame_rate );
 			}
 			else
 			{
@@ -1947,12 +1974,6 @@ static int video_codec_init( producer_avformat self, int index, mlt_properties p
 			}
 		}
 
-		// source_fps is deprecated in favor of meta.media.frame_rate_num and .frame_rate_den
-		if ( source_fps > 0 )
-			mlt_properties_set_double( properties, "source_fps", source_fps );
-		else
-			mlt_properties_set_double( properties, "source_fps", mlt_producer_get_fps( self->parent ) );
-
 		// Set the YUV colorspace from override or detect
 		self->colorspace = mlt_properties_get_int( properties, "force_colorspace" );
 #if LIBAVCODEC_VERSION_INT > ((52<<16)+(28<<8)+0)		
@@ -2050,11 +2071,8 @@ static void producer_set_up_video( producer_avformat self, mlt_frame frame )
 		// Set the width and height
 		mlt_properties_set_int( frame_properties, "width", self->video_codec->width );
 		mlt_properties_set_int( frame_properties, "height", self->video_codec->height );
-		// real_width and real_height are deprecated in favor of meta.media.width and .height
 		mlt_properties_set_int( properties, "meta.media.width", self->video_codec->width );
 		mlt_properties_set_int( properties, "meta.media.height", self->video_codec->height );
-		mlt_properties_set_int( frame_properties, "real_width", self->video_codec->width );
-		mlt_properties_set_int( frame_properties, "real_height", self->video_codec->height );
 		mlt_properties_set_double( frame_properties, "aspect_ratio", aspect_ratio );
 		mlt_properties_set_int( frame_properties, "colorspace", self->colorspace );
 
@@ -2062,7 +2080,6 @@ static void producer_set_up_video( producer_avformat self, mlt_frame frame )
 		if ( self->video_codec->height == 1088 && mlt_profile_dar( mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) ) ) == 16.0/9.0 )
 		{
 			mlt_properties_set_int( properties, "meta.media.height", 1080 );
-			mlt_properties_set_int( frame_properties, "real_height", 1080 );
 		}
 
 		// Add our image operation
