@@ -130,8 +130,6 @@ static uint8_t *filter_get_alpha_mask( mlt_frame this )
 		mlt_frame_set_alpha( this, alpha, region_width * region_height, NULL );
 	}
 
-	this->get_alpha_mask = NULL;
-
 	return alpha;
 }
 
@@ -282,7 +280,7 @@ static int transition_get_image( mlt_frame frame, uint8_t **image, mlt_image_for
 		// Index to hold the count
 		int i = 0;
 
-		// We will get the 'b frame' from the composite only if it's NULL
+		// We will get the 'b frame' from the composite only if it's NULL (region filter)
 		if ( b_frame == NULL )
 		{
 			// Copy the region
@@ -293,6 +291,13 @@ static int transition_get_image( mlt_frame frame, uint8_t **image, mlt_image_for
 			mlt_properties_set_data( MLT_FRAME_PROPERTIES( frame ), name, b_frame, 0, ( mlt_destructor )mlt_frame_close, NULL );
 		}
 
+		// filter_only prevents copying the alpha channel of the shape to the output frame
+		// by compositing filtered frame over itself
+		if ( mlt_properties_get_int( properties, "filter_only" ) )
+		{
+			frame = composite_copy_region( composite, b_frame, position );
+		}
+
 		// Make sure the filter is in the correct position
 		while ( filter != NULL )
 		{
