@@ -279,7 +279,6 @@ static int producer_collect_info( producer_libdv this, mlt_profile profile )
 					aspect_ratio = 8 / 9;
 			}
 			mlt_properties_set_double( properties, "aspect_ratio", aspect_ratio);
-			mlt_properties_set_double( properties, "source_fps", this->is_pal ? 25 : ( 30000.0 / 1001.0 ) );
 			mlt_properties_set_int( properties, "meta.media.nb_streams", 2 );
 			mlt_properties_set_int( properties, "video_index", 0 );
 			mlt_properties_set( properties, "meta.media.0.stream.type", "video" );
@@ -289,6 +288,10 @@ static int producer_collect_info( producer_libdv this, mlt_profile profile )
 			mlt_properties_set( properties, "meta.media.1.stream.type", "audio" );
 			mlt_properties_set( properties, "meta.media.1.codec.name", "pcm_s16le" );
 			mlt_properties_set( properties, "meta.media.1.codec.long_name", "signed 16-bit little-endian PCM" );
+			mlt_properties_set_int( properties, "meta.media.width", 720 );
+			mlt_properties_set_int( properties, "meta.media.height", this->is_pal ? 576 : 480 );
+			mlt_properties_set_int( properties, "meta.media.frame_rate_num", this->is_pal? 25 : 30000 );
+			mlt_properties_set_int( properties, "meta.media.frame_rate_den", this->is_pal?  1 :  1001 );
 
 			// Return the decoder
 			dv_decoder_return( dv_decoder );
@@ -335,7 +338,7 @@ static int producer_get_image( mlt_frame this, uint8_t **buffer, mlt_image_forma
 	*height = dv_data[ 3 ] & 0x80 ? 576 : 480;
 
 	// Extract an image of the format requested
-	if ( *format == mlt_image_yuv422 || *format == mlt_image_yuv420p )
+	if ( *format != mlt_image_rgb24 )
 	{
 		// Allocate an image
 		uint8_t *image = mlt_pool_alloc( *width * ( *height + 1 ) * 2 );
@@ -503,8 +506,6 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		// Update other info on the frame
 		mlt_properties_set_int( properties, "width", 720 );
 		mlt_properties_set_int( properties, "height", this->is_pal ? 576 : 480 );
-		mlt_properties_set_int( properties, "real_width", 720 );
-		mlt_properties_set_int( properties, "real_height", this->is_pal ? 576 : 480 );
 		mlt_properties_set_int( properties, "top_field_first", !this->is_pal ? 0 : ( data[ 5 ] & 0x07 ) == 0 ? 0 : 1 );
 		mlt_properties_set_int( properties, "colorspace", 601 );
 	
