@@ -154,6 +154,7 @@ mlt_producer producer_pango_init( const char *filename )
 		mlt_properties_set( properties, "style", "normal" );
 		mlt_properties_set( properties, "encoding", "UTF-8" );
 		mlt_properties_set_int( properties, "weight", PANGO_WEIGHT_NORMAL );
+		mlt_properties_set_int( properties, "seekable", 1 );
 
 		if ( filename == NULL || ( filename && ( !strcmp( filename, "" )
 			// workaround for old kdenlive countdown generator
@@ -442,8 +443,8 @@ static void refresh_image( mlt_frame frame, int width, int height )
 			g_object_ref( pixbuf );
 			mlt_properties_set_data( MLT_FRAME_PROPERTIES( frame ), "pixbuf", pixbuf, 0, ( mlt_destructor )g_object_unref, NULL );
 
-			mlt_properties_set_int( producer_props, "real_width", gdk_pixbuf_get_width( pixbuf ) );
-			mlt_properties_set_int( producer_props, "real_height", gdk_pixbuf_get_height( pixbuf ) );
+			mlt_properties_set_int( producer_props, "meta.media.width", gdk_pixbuf_get_width( pixbuf ) );
+			mlt_properties_set_int( producer_props, "meta.media.height", gdk_pixbuf_get_height( pixbuf ) );
 
 			// Store the width/height of the pixbuf temporarily
 			this->width = gdk_pixbuf_get_width( pixbuf );
@@ -484,8 +485,6 @@ static void refresh_image( mlt_frame frame, int width, int height )
 	// Set width/height
 	mlt_properties_set_int( properties, "width", this->width );
 	mlt_properties_set_int( properties, "height", this->height );
-	mlt_properties_set_int( properties, "real_width", mlt_properties_get_int( producer_props, "real_width" ) );
-	mlt_properties_set_int( properties, "real_height", mlt_properties_get_int( producer_props, "real_height" ) );
 }
 
 static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_format *format, int *width, int *height, int writable )
@@ -508,6 +507,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	// Get width and height
 	*width = this->width;
 	*height = this->height;
+	*format = mlt_image_rgb24a;
 
 	// Always clone here to allow 'animated' text
 	if ( this->pixbuf )
@@ -519,7 +519,6 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 
 		// Now update properties so we free the copy after
 		mlt_frame_set_image( frame, *buffer, image_size, mlt_pool_release );
-		*format = mlt_image_rgb24a;
 	}
 	else
 	{
@@ -536,6 +535,9 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 {
 	producer_pango this = producer->child;
 
+	// Fetch the producers properties
+	mlt_properties producer_properties = MLT_PRODUCER_PROPERTIES( producer );
+
 	// Generate a frame
 	*frame = mlt_frame_init( MLT_PRODUCER_SERVICE( producer ) );
 
@@ -556,7 +558,11 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 
 	// Set producer-specific frame properties
 	mlt_properties_set_int( properties, "progressive", 1 );
-	mlt_properties_set_double( properties, "aspect_ratio", 1 );
+	double force_ratio = mlt_properties_get_double( producer_properties, "force_aspect_ratio" );
+	if ( force_ratio > 0.0 )
+		mlt_properties_set_double( properties, "aspect_ratio", force_ratio );
+	else
+		mlt_properties_set_double( properties, "aspect_ratio", 1.0);
 
 	// Stack the get image callback
 	mlt_frame_push_service( *frame, this );
