@@ -152,7 +152,6 @@ static int framebuffer_get_image( mlt_frame frame, uint8_t **image, mlt_image_fo
         uint8_t *first_alpha = mlt_properties_get_data( first_frame_properties, "alpha", NULL );
 	if ( !first_image )
 	{
-		mlt_properties_set_double( first_frame_properties, "consumer_aspect_ratio", mlt_properties_get_double( frame_properties, "consumer_aspect_ratio" ) );
 		mlt_properties_set( first_frame_properties, "rescale.interp", mlt_properties_get( frame_properties, "rescale.interp" ) );
 
 		int error = mlt_frame_get_image( first_frame, &first_image, format, width, height, writable );
@@ -238,8 +237,8 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		// Give the returned frame temporal identity
 		mlt_frame_set_position( *frame, mlt_producer_position( producer ) );
 
-		mlt_properties_set_int( frame_properties, "real_width", mlt_properties_get_int( properties, "width" ) );
-		mlt_properties_set_int( frame_properties, "real_height", mlt_properties_get_int( properties, "height" ) );
+		mlt_properties_set_int( frame_properties, "meta.media.width", mlt_properties_get_int( properties, "width" ) );
+		mlt_properties_set_int( frame_properties, "meta.media.height", mlt_properties_get_int( properties, "height" ) );
 		mlt_properties_pass_list( frame_properties, properties, "width, height" );
 	}
 
