@@ -32,6 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <ctype.h>
 
 static void load_filenames( producer_qimage self, mlt_properties producer_properties );
 static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int index );
@@ -59,7 +60,8 @@ mlt_producer producer_qimage_init( mlt_profile profile, mlt_service_type type, c
 		mlt_properties_set_int( properties, "ttl", 25 );
 		mlt_properties_set_int( properties, "aspect_ratio", 1 );
 		mlt_properties_set_int( properties, "progressive", 1 );
-		
+		mlt_properties_set_int( properties, "seekable", 1 );
+
 		// Validate the resource
 		if ( filename )
 			load_filenames( self, properties );
@@ -72,7 +74,8 @@ mlt_producer producer_qimage_init( mlt_profile profile, mlt_service_type type, c
 				mlt_properties_set_data( frame_properties, "producer_qimage", self, 0, NULL, NULL );
 				mlt_frame_set_position( frame, mlt_producer_position( producer ) );
 				mlt_properties_set_position( frame_properties, "qimage_position", mlt_producer_position( producer ) );
-				refresh_qimage( self, frame, 0, 0 );
+				refresh_qimage( self, frame );
+				mlt_cache_item_close( self->qimage_cache );
 				mlt_frame_close( frame );
 			}
 		}
@@ -81,29 +84,34 @@ mlt_producer producer_qimage_init( mlt_profile profile, mlt_service_type type, c
 			producer_close( producer );
 			producer = NULL;
 		}
-		if ( producer )
-			pthread_mutex_init( &self->mutex, NULL );
 		return producer;
 	}
 	free( self );
 	return NULL;
 }
 
-static void load_filenames( producer_qimage self, mlt_properties producer_properties )
+static int load_svg( producer_qimage self, mlt_properties properties, const char *filename )
 {
-	char *filename = mlt_properties_get( producer_properties, "resource" );
-	self->filenames = mlt_properties_new( );
+	int result = 0;
 
 	// Read xml string
 	if ( strstr( filename, "<svg" ) )
 	{
 		make_tempfile( self, filename );
+		result = 1;
 	}
-	// Obtain filenames
-	else if ( strchr( filename, '%' ) != NULL )
+	return result;
+}
+
+static int load_sequence( producer_qimage self, mlt_properties properties, const char *filename )
+{
+	int result = 0;
+
+	// Obtain filenames with pattern
+	if ( strchr( filename, '%' ) != NULL )
 	{
 		// handle picture sequences
-		int i = mlt_properties_get_int( producer_properties, "begin" );
+		int i = mlt_properties_get_int( properties, "begin" );
 		int gap = 0;
 		char full[1024];
 		int keyvalue = 0;
@@ -125,9 +133,47 @@ static void load_filenames( producer_qimage self, mlt_properties producer_proper
 			}
 		}
 		if ( mlt_properties_count( self->filenames ) > 0 )
-			mlt_properties_set_int( producer_properties, "ttl", 1 );
+		{
+			mlt_properties_set_int( properties, "ttl", 1 );
+			result = 1;
+		}
 	}
-	else if ( strstr( filename, "/.all." ) != NULL )
+	return result;
+}
+
+static int load_sequence2( producer_qimage self, mlt_properties properties, const char *filename )
+{
+	int result = 0;
+	const char *start;
+
+	// Obtain filenames with pattern containing a begin value, e.g. foo%1234d.png
+	if ( ( start = strchr( filename, '%' ) ) )
+	{
+		const char *end = ++start;
+		while ( isdigit( *end ) ) end++;
+		if ( end > start && ( end[0] == 'd' || end[0] == 'i' || end[0] == 'u' ) )
+		{
+			int n = end - start;
+			char *s = calloc( 1, n + 1 );
+			strncpy( s, start, n );
+			mlt_properties_set( properties, "begin", s );
+			free( s );
+			s = calloc( 1, strlen( filename ) );
+			strncpy( s, filename, start - filename );
+			sprintf( s + ( start - filename ), ".%d%s", n, end );
+			result = load_sequence( self, properties, s );
+			free( s );
+		}
+	}
+	return result;
+}
+
+static int load_folder( producer_qimage self, mlt_properties properties, const char *filename )
+{
+	int result = 0;
+
+	// Obtain filenames within folder
+	if ( strstr( filename, "/.all." ) != NULL )
 	{
 		char wildcard[ 1024 ];
 		char *dir_name = strdup( filename );
@@ -139,12 +185,23 @@ static void load_filenames( producer_qimage self, mlt_properties producer_proper
 		mlt_properties_dir_list( self->filenames, dir_name, wildcard, 1 );
 
 		free( dir_name );
+		result = 1;
 	}
-	else
+	return result;
+}
+
+static void load_filenames( producer_qimage self, mlt_properties properties )
+{
+	char *filename = mlt_properties_get( properties, "resource" );
+	self->filenames = mlt_properties_new( );
+
+	if (!load_svg( self, properties, filename ) &&
+		!load_sequence( self, properties, filename ) &&
+		!load_sequence2( self, properties, filename ) &&
+		!load_folder( self, properties, filename ) )
 	{
 		mlt_properties_set( self->filenames, "0", filename );
 	}
-
 	self->count = mlt_properties_count( self->filenames );
 }
 
@@ -152,11 +209,10 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 {
 	int error = 0;
 	
-	// Obtain properties of frame
+	// Obtain properties of frame and producer
 	mlt_properties properties = MLT_FRAME_PROPERTIES( frame );
-
-	// Obtain the producer for this frame
 	producer_qimage self = mlt_properties_get_data( properties, "producer_qimage", NULL );
+	mlt_producer producer = &self->parent;
 
 	*width = mlt_properties_get_int( properties, "rescale_width" );
 	*height = mlt_properties_get_int( properties, "rescale_height" );
@@ -164,19 +220,25 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	mlt_service_lock( MLT_PRODUCER_SERVICE( &self->parent ) );
 
 	// Refresh the image
-	refresh_qimage( self, frame, *width, *height );
+	self->qimage_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.qimage" );
+	self->qimage = mlt_cache_item_data( self->qimage_cache, NULL );
+	self->image_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.image" );
+	self->current_image = mlt_cache_item_data( self->image_cache, NULL );
+	self->alpha_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.alpha" );
+	self->current_alpha = mlt_cache_item_data( self->alpha_cache, NULL );
+	refresh_image( self, frame, *format, *width, *height );
 
 	// Get width and height (may have changed during the refresh)
 	*width = mlt_properties_get_int( properties, "width" );
 	*height = mlt_properties_get_int( properties, "height" );
-	*format = self->has_alpha ? mlt_image_rgb24a : mlt_image_rgb24;
+	*format = self->format;
 
 	// NB: Cloning is necessary with this producer (due to processing of images ahead of use)
 	// The fault is not in the design of mlt, but in the implementation of the qimage producer...
 	if ( self->current_image )
 	{
 		// Clone the image and the alpha
-		int image_size = self->current_width * ( self->current_height + 1 ) * ( self->has_alpha ? 4 :3 );
+		int image_size = mlt_image_format_size( self->format, self->current_width, self->current_height, NULL );
 		uint8_t *image_copy = mlt_pool_alloc( image_size );
 		memcpy( image_copy, self->current_image, image_size );
 		// Now update properties so we free the copy after
@@ -185,6 +247,13 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		*buffer = image_copy;
 		mlt_log_debug( MLT_PRODUCER_SERVICE( &self->parent ), "%dx%d (%s)\n",
 			self->current_width, self->current_height, mlt_image_format_name( *format ) );
+		// Clone the alpha channel
+		if ( self->current_alpha )
+		{
+			image_copy = mlt_pool_alloc( self->current_width * self->current_height );
+			memcpy( image_copy, self->current_alpha, self->current_width * self->current_height );
+			mlt_frame_set_alpha( frame, image_copy, self->current_width * self->current_height, mlt_pool_release );
+		}
 	}
 	else
 	{
@@ -192,8 +261,9 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 	}
 
 	// Release references and locks
-	pthread_mutex_unlock( &self->mutex );
+	mlt_cache_item_close( self->qimage_cache );
 	mlt_cache_item_close( self->image_cache );
+	mlt_cache_item_close( self->alpha_cache );
 	mlt_service_unlock( MLT_PRODUCER_SERVICE( &self->parent ) );
 
 	return error;
@@ -228,7 +298,10 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		mlt_properties_set_position( properties, "qimage_position", mlt_producer_position( producer ) );
 
 		// Refresh the image
-		refresh_qimage( self, *frame, 0, 0 );
+		self->qimage_cache = mlt_service_cache_get( MLT_PRODUCER_SERVICE( producer ), "qimage.qimage" );
+		self->qimage = mlt_cache_item_data( self->qimage_cache, NULL );
+		refresh_qimage( self, *frame );
+		mlt_cache_item_close( self->qimage_cache );
 
 		// Set producer-specific frame properties
 		mlt_properties_set_int( properties, "progressive", mlt_properties_get_int( producer_properties, "progressive" ) );
@@ -251,7 +324,6 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 static void producer_close( mlt_producer parent )
 {
 	producer_qimage self = parent->child;
-	pthread_mutex_destroy( &self->mutex );
 	parent->close = NULL;
 	mlt_service_cache_purge( MLT_PRODUCER_SERVICE(parent) );
 	mlt_producer_close( parent );
