@@ -163,7 +163,7 @@ static int consumer_start( mlt_consumer parent )
 
 		// Default window size
 		double display_ratio = mlt_properties_get_double( this->properties, "display_ratio" );
-		this->window_width = ( double )this->height * display_ratio;
+		this->window_width = ( double )this->height * display_ratio + 0.5;
 		this->window_height = this->height;
 
 		if ( sdl_started == 0 && preview_off == 0 )
@@ -614,10 +614,11 @@ static int consumer_get_dimensions( int *width, int *height )
 	// Specify the SDL Version
 	SDL_VERSION( &wm.version );
 
+#ifndef __DARWIN__
 	// Get the wm structure
 	if ( SDL_GetWMInfo( &wm ) == 1 )
 	{
-#ifndef __DARWIN__
+#ifndef WIN32
 		// Check that we have the X11 wm
 		if ( wm.subsystem == SDL_SYSWM_X11 ) 
 		{
@@ -640,6 +641,7 @@ static int consumer_get_dimensions( int *width, int *height )
 		}
 #endif
 	}
+#endif
 
 	return changed;
 }
