@@ -171,8 +171,8 @@ static int producer_open( mlt_producer this, mlt_profile profile, char *file )
 
 				// Get the vorbis info
 				vorbis_info *vi = ov_info( ov, -1 );
-				mlt_properties_set_int( properties, "frequency", (int) vi->rate );
-				mlt_properties_set_int( properties, "channels", vi->channels );
+				mlt_properties_set_int( properties, "audio_frequency", (int) vi->rate );
+				mlt_properties_set_int( properties, "audio_channels", vi->channels );
 
 				// Set some media metadata
 				mlt_properties_set_int( properties, "meta.media.nb_streams", 1 );
